Struct CellVoltage
Source pub struct CellVoltage {
pub cell_1_millivolts: u16,
pub cell_2_millivolts: u16,
pub cell_3_millivolts: u16,
pub cell_4_millivolts: u16,
}
Voltage readings for each of the four battery cells.
Formats the value using the given formatter.
Read more
Immutably borrows from an owned value.
Read more
Mutably borrows from an owned value.
Read more
Returns the argument unchanged.
Calls U::from(self).
That is, this conversion is whatever the implementation of
From<T> for U chooses to do.
The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.